<?php

namespace Drupal\self_evaluation\Plugin\self_evaluation\NoteType;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\self_evaluation\NoteType\NoteTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Abstract Class for NoteType Plugin.
 */
abstract class NoteType extends PluginBase implements NoteTypeInterface, ContainerFactoryPluginInterface {

  /**
   * Entity type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  public $entityTypeManager;

  /**
   * The max base total under which we want to calculate the score.
   *
   * @var int
   */
  protected $maxBase = 0;

  /**
   * Class constructor.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): NoteType {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function description() {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function calculate(float $note, float $max): array {
    if ($max <= 0) {
      return [];
    }
    if (empty($this->maxBase)) {
      return [
        'note' => $note,
        'max' => $max,
      ];
    }
    else {
      return [
        'note' => ($note * $this->maxBase) / $max,
        'max' => $this->maxBase,
      ];
    }

  }

}
